/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.webidor.resources;

import cz.xelfi.quoridor.IllegalPositionException;
import cz.xelfi.quoridor.webidor.*;
import cz.xelfi.quoridor.Move;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public final class Games {
    private List<Game> games = new ArrayList<Game>();
    private final File dir;
    private static final Logger LOG = Logger.getLogger(Games.class.getName());

    public Games(File dir) {
        this.dir = dir;
        File[] arr = dir.listFiles();
        if (arr != null) {
            for (File f : arr) {
                try {
                    Game g = readGame(f);
                    games.add(g);
                } catch (IOException ex) {
                    LOG.log(Level.WARNING, "Wrong game in " + f, ex);
                }
            }
        }
    }

    @POST
    @Produces(MediaType.APPLICATION_JSON)
    public Game createGame(
        @QueryParam("white") String white,
        @QueryParam("black") String black
    ) {
        if (white == null) {
            throw new IllegalArgumentException("Must specify white");
        }
        if (black == null) {
            throw new IllegalArgumentException("Must specify black");
        }
        Game g = new Game(white, black);
        games.add(g);
        return g;
    }

    @GET
    @Path("{id}")
    @Produces(MediaType.TEXT_PLAIN)
    public String getBoard(@PathParam("id") String id) {
        Game g = findGame(id);
        if (g == null) {
            throw new IllegalArgumentException("Unknown game " + id);
        }
        return g.getBoard().toString();
    }

    @PUT
    @Path("{id}")
    @Produces(MediaType.APPLICATION_JSON)
    public Game applyMove(
        @PathParam("id") String id,
        @QueryParam("player") String player,
        @QueryParam("move") String move
    ) throws IllegalPositionException {
        Game g = findGame(id);
        if (g == null) {
            throw new IllegalArgumentException("Unknown game " + id);
        }
        Move m = Move.valueOf(move);
        g.apply(player, m);
        try {
            storeGame(g);
        } catch (IOException ex) {
            LOG.log(Level.WARNING, "Cannot store game " + id, ex);
        }
        return g;
    }

    @GET
    @Produces(MediaType.APPLICATION_JSON)
    public List<Game> getGames() {
        return games;
    }

    private Game findGame(String id) {
        for (Game g : games) {
            if (g.getId().equals(id)) {
                return g;
            }
        }
        return null;
    }

    private Game readGame(File f) throws IOException {
        BufferedReader r = new BufferedReader(new FileReader(f));
        String white = null;
        String black = null;
        Game g = null;
        for (;;) {
            String line = r.readLine();
            if (line == null) {
                break;
            }
            line = line.trim();
            if (line.length() == 0) {
                continue;
            }
            if (line.startsWith("# white: ")) {
                white = line.substring(9);
                continue;
            }
            if (line.startsWith("# black: ")) {
                black = line.substring(9);
                continue;
            }
            if (line.startsWith("#")) {
                continue;
            }
            if (white == null || black == null) {
                throw new IOException("Missing white and black identification in " + f);
            }
            if (g == null) {
                g = new Game(f.getName(), white, black);
            }
            String[] moves = line.split(" ");
            if (moves.length == 0) {
                continue;
            }
            if (moves.length > 2) {
                throw new IOException("Too much moves on line: " + line);
            }
            try {
                g.apply(white, Move.valueOf(moves[0]));
                if (moves.length == 2) {
                    g.apply(black, Move.valueOf(moves[1]));
                }
            } catch (IllegalPositionException ex) {
                throw new IOException("Wrong move: " + ex.getMessage());
            }
        }
        if (g == null) {
            throw new IOException("No moves in " + f);
        }
        return g;
    }

    private void storeGame(Game g) throws IOException {
        dir.mkdirs();
        File f = new File(dir, g.getId());
        PrintWriter pw = new PrintWriter(new BufferedWriter(new FileWriter(f)));
        pw.println("# white: " + g.getWhite());
        pw.println("# black: " + g.getBlack());
        int cnt = 0;
        for (Move m : g.getMoves()) {
            pw.print(m.toString());
            if (++cnt % 2 == 0) {
                pw.println();
            } else {
                pw.print(' ');
            }
        }
        pw.println();
        pw.println();
        pw.flush();
        pw.close();
    }
}
